import argparse
import glob
import subprocess
import sys

import backtracepython as bt

from backtrace_unreald.unreald_config import UnrealdConfig
from backtrace_unreald.unreald_utils import log
from backtrace_unreald.unreald_webservice import run_app


def run_single(config, instance_id):
    unreald_lib_dir = glob.glob('../lib/*/site-packages/backtrace_unreald/')[0]
    while True:
        subprocess.call([sys.executable, '{0}/unreald_instance.py'.format(unreald_lib_dir),
                         "-http_port", str(config.http_port + instance_id),
                         "-config_file", config.config_file,
                         "-instance_id", str(instance_id)
                         ])


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='Crash Receiver')
    parser.add_argument('--config-file', '-c', type=str, default=None)
    args = parser.parse_args()

    if args.config_file is None:
        default_config_paths = [
            '/usr/local/etc/unreald/unreald.conf',
            '/etc/unreald/unreald.conf',
            '/etc/backtrace/unreald/unreald.conf',
            '/opt/backtrace/etc/unreald/unreald.conf',
        ]
    else:
        default_config_paths = [args.config_file]

    config = None
    for path in default_config_paths:
        try:
            config = UnrealdConfig(args, path)
            break
        except FileNotFoundError as e:
            log('Failed to open {}'.format(path))

    if config is None:
        log('Failed to load config')
        exit(1)
    bt.initialize(
        endpoint=config.script_error_reporting.backtrace_endpoint,
        token=config.script_error_reporting.backtrace_token,
    )

    run_app(config)
