import collections
import json
from copy import deepcopy


def update_dict(d, u):
    for k, v in u.items():
        if isinstance(v, collections.Mapping):
            base = d.get(k, {})
            if base is None:
                base = {}
            r = update_dict(base, v)
            d[k] = r
        elif isinstance(v, list):
            base = d.get(k, [])
            if len(base) == 0:
                base = None
            if len(v) == 0 or isinstance(v[0], dict):
                if base is None:
                    d[k] = [update_dict({}, el) for el in v]
                else:
                    d[k] = [update_dict(deepcopy(base[0]), el) for el in v]
            else:
                d[k] = v
        else:
            d[k] = u[k]
    return d


class ConfigNode:
    def __init__(self, **response):
        for k, v in response.items():
            if isinstance(v, dict):
                self.__dict__[k] = ConfigNode(**v)
            elif isinstance(v, list):
                if len(v) == 0 or isinstance(v, dict):
                    self.__dict__[k] = [ConfigNode(**el) for el in v]
                else:
                    self.__dict__[k] = v
            else:
                self.__dict__[k] = v

    def dict(self):
        return self.__dict__



class UnrealdConfig:

    def __init__(self, args, config_file):

        d = {
            'http': [{
                'address': '0.0.0.0',
                'listen_port': 6100,
                'ssl': None,
                'listen_backlog': 128,
            }],
            "max_request_size": 104857600,
            'endpoint': {
                'url_format': ("https://{name}.{suffix}:{port}/post?"
                               "format=minidump&token={token}"),
                'port': 6098,
                'suffix': 'sp.backtrace.io',
                'custom_mapping': {},
            },
            'metrics': {
                'address': '0.0.0.0',
                'listen_port': 6100,
                'ssl': None,
                'listen_backlog': 128,
                'api_token': "",
            },
            'dump_dirs': '/var/unreald',
            'processes': 0,
            'script_error_reporting': {
                'backtrace_endpoint': ("https://yourusername.sp."
                                       "backtrace.io:6098"),
                'backtrace_token': "setyourowntoken",
            },
            'blacklist': [],
            'run_as': {
                'username': 'nobody',
                'group': 'nogroup',
            },
            'limits': {
                'max_concurrent_uploads': 4,
                'max_unsent_queue_size': 1024,
                'max_requests_per_second': 1000,
                'max_incoming_bytes_per_second': 1024 * 1024 * 25,
            },
            'retries': {
                'count': 5,
                'delay': 1,
                'exp': 2
            },
            'attributes': {
                'aliases': {
                    'MachineID': 'guid',
                    'SecondsSinceStart': 'process.age',
                    'ErrorMessage' : 'error.message',
                    'Misc.PrimaryGPUBrand' : 'gpu.brand',
                    'MemoryStats.UsedPhysical' : 'vm.rss.size',       # KB
                    'MemoryStats.PeakUsedPhysical' : 'vm.rss.peak',   # KB
                    'MemoryStats.UsedVirtual' : 'vm.vma.size',        # KB
                    'MemoryStats.PeakUsedVirtual' : 'vm.vma.peak',    # KB
                    'MemoryStats.TotalPhysical' : 'vm.rss.available', # KB
                    'MemoryStats.TotalVirtual' : 'vm.vma.available',  # KB
                },
                'ignore': [
                      'SourceContext'
                    , 'MemoryStats.PageSize'
                ],
            },
            "dump_last_request": False,
            "include_ini": False,
            "include_log": True,
        }

        with open(config_file) as file:
            update_dict(d, json.load(file))

        d['config_file'] = config_file

        for k, v in d.items():
            if isinstance(v, dict):
                self.__dict__[k] = ConfigNode(**v)
            elif isinstance(v, list):
                self.__dict__[k] = [ConfigNode(**el) for el in v]
            else:
                self.__dict__[k] = v

        self.attributes.ignore = set(self.attributes.ignore)


    def dict(self):
        return self.__dict__


    def __setattr__(self, name, value):
        raise Exception("Config is read only!")
