import asyncio
import mimetypes

import requests
from requests_toolbelt import MultipartEncoder


def get_attribute_fields(attributes):
    return [(k, str(v)) for k,v in attributes.items()]

def get_file_fields(files):
    for formname, filename, file in files:
        mtype = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
        yield (formname, (filename, open(file, 'rb'), mtype))

def post(url, attributes, files):
    fields = list(get_attribute_fields(attributes)) + list(get_file_fields(files))

    data = MultipartEncoder(fields)
    headers = {'Content-Type': data.content_type}

    return requests.post(url, data=data, headers=headers)

def post_async(url, attributes, files, loop=None):
    return (asyncio.get_event_loop() if loop is None else loop).run_in_executor(None, post, url, attributes, files)
