import base64
import re

import xmltodict

import backtrace_unreald.unreald_utils as uu
from backtrace_unreald.unreald_utils import log

TO_KB = set(['MemoryStats.UsedPhysical',
             'MemoryStats.PeakUsedPhysical',
             'MemoryStats.UsedVirtual',
             'MemoryStats.PeakUsedVirtual',
             'MemoryStats.TotalPhysical',
             'MemoryStats.TotalVirtual',
            ])

def extract_attributes_from_xml_default(xml, conf):
    result = {}
    for k, v in xml.items():
        if k in conf.ignore:
            continue
        if v is None:
            continue
        if k == 'MemoryStats.TotalPhysicalGB':
            v = int(v) * 1024**2
        if k in TO_KB:
            v = int(int(v) / int(1024))
        result[conf.aliases.dict().get(k, k)] = v

    return result

def extract_attributes_from_xml_fallback(xml_string, conf, node_name):
    log('fallback xml parser')
    print(xml_string)
    result = {}

    m = re.compile('<%s>.*?</%s>' % (node_name, node_name), re.S).search(xml_string)
    print(m)
    if m is not None:
        try:
            print(m.group(0))
            xml_data = xmltodict.parse(m.group(0))
            result = extract_attributes_from_xml_default(xml_data, conf)
        except BaseException as e:
            print('except: ', e)
            pass

    return result

@uu.report_to_backtrace
def extract_attributes_from_xml(xml, conf):
    extracted = {}
    def to_base64(match):
        inner = match.group(2).encode()
        outer = match.group(1)
        inner = base64.b64encode(inner).decode('utf-8')
        return '<%s>%s</%s>' % (outer, inner, outer)

    xml = re.compile('<CallStack>.*?</CallStack>', re.S).sub('', xml)
    xml = re.compile('<Modules>.*?</Modules>', re.S).sub('', xml)
    xml = re.sub('<(SourceContext)>(.*?)</SourceContext>', to_base64, xml, flags=re.S)

    xml = re.sub('&nl;', '%%newline%%', xml)

    def extract(node_name):
        ret = {}
        try:
            xml_data = xmltodict.parse(xml)
            xml_data = xml_data\
                .get('FGenericCrashContext', {})\
                .get(node_name, {})
            ret = extract_attributes_from_xml_default(xml_data, conf)
        except:
            try:
                ret = extract_attributes_from_xml_fallback(xml, conf, node_name)
            except:
                pass
        return ret

    extracted = extract('RuntimeProperties')
    extracted.update(extract('GameData'))

    return extracted

